/*
 * Copyright (c) 2025 Adriaan van Wijk
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "dm_data.h"
#include "claude.h"
#include <zephyr/bluetooth/addr.h>
#include <zephyr/sys/printk.h>
#include <zephyr/bluetooth/bluetooth.h>


// Global variables
struct adv_mfg_data mfg_data;

// Global array to store distance data from all devices
struct distance_data peer_distances[5] = {
    {.device_id = 1, .valid = false},
    {.device_id = 2, .valid = false},
    {.device_id = 3, .valid = false},
    {.device_id = 4, .valid = false},
    {.device_id = 5, .valid = false}
};

// Convert meters to centimeters
static uint16_t meters_to_cm(float meters) {
    return (uint16_t)(meters * 100.0f);
}

struct adv_mfg_data* get_mfg_data(void) {
    return &mfg_data;
}


void update_mfg_distances(uint8_t from_id, uint8_t to_id, uint16_t distance) {
    // Ensure valid device IDs (1-5)
    if (from_id < 1 || from_id > 5 || to_id < 1 || to_id > 5) {
        return;
    }
    
    // Get pointer to our mfg data
    struct adv_mfg_data* our_data = get_mfg_data();
    
    // Update the appropriate distance field based on to_id
    switch (to_id) {
        case 1:
            our_data->dist_to_1 = distance;
            break;
        case 2:
            our_data->dist_to_2 = distance;
            break;
        case 3:
            our_data->dist_to_3 = distance;
            break;
        case 4:
            our_data->dist_to_4 = distance;
            break;
        case 5:
            our_data->dist_to_5 = distance;
            break;
    }
}

void read_mfg_data(const struct adv_mfg_data *recv_mfg) {
    if (!recv_mfg) {
        return;
    }

    // Get the device ID from the received data
    uint8_t device_id = recv_mfg->device_id;
    
    // Ensure valid device ID
    if (device_id < 1 || device_id > 5) {
        return;
    }

    // Store all distances from this device
    struct distance_data *peer_data = &peer_distances[device_id - 1];
    peer_data->device_id = device_id;
    peer_data->valid = true;
    
    // Store all the distances this peer knows about
    peer_data->distances[0] = recv_mfg->dist_to_1;
    peer_data->distances[1] = recv_mfg->dist_to_2;
    peer_data->distances[2] = recv_mfg->dist_to_3;
    peer_data->distances[3] = recv_mfg->dist_to_4;
    peer_data->distances[4] = recv_mfg->dist_to_5;

    // Print the received data
    printk("\nReceived data from Device %d:", device_id);
    for (int i = 0; i < 5; i++) {
        if (peer_data->distances[i] == 0xFFFF) {
            printk("\n  Distance to Device %d: No Data", i + 1);
        } else {
            printk("\n  Distance to Device %d: %d cm", i + 1, peer_data->distances[i]);
        }
    }
    printk("\n");
}



const struct adv_mfg_data* get_peer_mfg_data(const bt_addr_le_t *peer_addr) {
    // The manufacturer data is in the scan response data
    // In the data_cb function in main.c, we already parse this and 
    // access it as recv_mfg_data
    // For now, we'll use the data that's already in our scan response
    return &mfg_data;
}



//This assumes that the DISTANCE is for the LAST packet recieved.
void update_mfg_data(float distance) {
    const struct adv_mfg_data* peer_mfg = get_last_recv_mfg_data();
    if (!peer_mfg) {
        return;
    }
   
    // Convert the measured distance from meters to centimeters
    uint16_t dist_cm = meters_to_cm(distance);
   
    // Get the actual device ID from the manufacturer data
    uint8_t peer_device_id = peer_mfg->device_id;
   
    // Update our manufacturer data for this peer
    update_mfg_distances(mfg_data.device_id, peer_device_id, dist_cm);
   
    // Print the updated measurement
    printk("\nUpdated measurement:");
    printk("\n  From Device %d to Device %d: %d cm\n",
           mfg_data.device_id, peer_device_id, dist_cm);
}